
# q-expansion of newform 1110.4.a.g, downloaded from the LMFDB on 15 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 1110
weight = 4
poly_data = [-1, -6, 0, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0, 0], 1], [[0, 1, 0], 1], [[-4, 0, 1], 1]]

hecke_ring_character_values = None
aps_data = [[-2, 0, 0], [3, 0, 0], [5, 0, 0], [-4, 1, 3], [3, 5, 10], [-11, 0, -2], [-35, 12, -11], [-27, -26, -17], [-20, -47, -29], [-36, 5, -50], [-114, 47, -21], [37, 0, 0], [-200, -125, 31], [30, 99, -26], [-79, -140, 86], [21, 153, 39], [-88, 156, -165], [-244, -33, 29], [-289, -48, 49], [-411, 220, 135], [-279, -429, 152], [-351, -191, 200], [104, 284, -63], [165, -388, 134], [-364, 401, -13], [-149, 344, 367], [-301, -18, -370], [955, 393, 61], [239, 444, -87], [-367, -357, 316], [629, -289, -260], [340, 29, -595], [1194, -65, 502], [524, -629, 548], [1440, -657, 171], [-1507, 500, 260], [-1230, -119, -39], [-1786, 364, -270], [67, -1218, 581], [185, 204, -780], [-877, 1036, -568], [-1212, -670, 733], [184, -1188, -776], [-2427, 1301, 319], [1108, 2269, -605], [-1188, 1279, 17], [122, 810, -257], [-1394, -1321, -443], [1124, -748, 320], [-3941, -137, -133], [-584, 1489, 759], [-1079, 1326, -1118], [-2327, 1299, -392], [-248, -2263, 558], [-647, 60, 283], [346, -633, -856], [-4656, -139, 839], [-2699, -1913, 1075], [-1742, 1134, -970], [3022, 925, -970], [-1047, 2454, 597], [614, -2057, -233], [2482, -13, 1841], [-599, 2855, -687], [-1714, -2511, 1006], [1675, -133, 1671], [-2189, -2122, 307], [4102, -3539, -1637], [7674, -1176, 1082], [-1198, 3105, -1546], [-3526, -805, -959], [1412, -765, 1899], [1355, 3468, -1927], [5597, 740, -1831], [-6640, 1247, -2596], [5539, -713, -776], [-789, 43, -2967], [-1628, -3335, 751], [6883, -1829, -1585], [-663, -3644, 153], [1008, 4702, -754], [6048, -1469, 1053], [510, 156, 3042], [1427, 3330, 1515], [-1802, -2584, 2544], [7537, 2246, 316], [5799, -1827, 328], [125, 1093, -2041], [4977, 1488, 1620], [5319, -603, 2623], [589, 1853, -658], [-3477, -3162, 1678], [-44, 3770, -1693], [4940, -2787, -1565], [-6954, -2846, 142], [774, 1002, 5038], [-11872, -480, 1416], [6825, -1866, 1979], [7621, 2373, 360], [-480, -1695, -2145], [13243, -1241, 4387], [12851, -1757, 3768], [3489, -5304, -2041], [6657, -1604, -2567], [-2198, -3139, 2015], [7263, -332, 3247], [3000, 3099, -2727], [-2121, -1696, -6844], [-11984, 593, 2181], [7559, 3297, -1449], [5568, -7475, -1451], [8484, -5018, 1832], [-3663, -5799, 2626], [-5062, -1284, 742], [9025, 673, 346], [-12184, -7081, 675], [10382, 1290, -4505], [-7658, 1384, -10036], [4254, 9414, -1638], [558, -8925, 2503], [9642, 5395, 3533], [-5765, -594, -3037], [10244, -1979, 4364], [8570, 2166, 1836], [6467, 3764, -2152], [-4175, 13161, 107], [-17620, -1010, 4338], [-6436, -675, -861], [4243, -8152, 4997], [-880, -99, -4471], [-8716, 4315, -664], [18751, -609, 5072], [-4743, -12013, -94], [5870, 1307, 6420], [-17373, 4127, -148], [-22523, 2980, 4851], [-6110, -8740, 6536], [-3581, 9805, -1274], [-8372, 1251, -11359], [1686, 8320, -89], [21207, -4960, 345], [-13578, -8643, 2391], [-160, -13348, 4522], [-17922, -471, 5685], [-24150, -1072, -3268], [-7472, 13091, -757], [-17403, 2253, -12651], [-3298, 4285, 1623], [-14344, -5798, -3330], [2120, 2320, 2541], [-13407, -4047, -9966], [-13677, -5490, -6329], [10444, -47, -4047], [-24636, -7575, -999], [28265, -5057, 6303], [2281, 12276, 5006], [-11692, -3992, -5094], [-16062, -8230, -7662], [199, 3508, 10739], [-221, 4517, 6860], [22760, 3696, 8742], [-13368, 2998, -13312], [21825, 9953, -1852], [19426, 14805, -5855], [-6405, 4639, -11828], [-2541, 2856, 1009], [-3661, 13847, -1622], [-17605, -1451, -6352], [-719, -4950, 8838], [-4842, -14505, -1899], [16428, -3631, 5797], [-12797, 1823, -4526], [-20740, -4499, 4414], [21001, 2713, 14486], [15201, 4333, -5936], [-2312, 55, 6289], [44978, -3590, 2820], [-29010, -285, -2461], [6868, -9106, -1214], [14333, -10068, -7980], [14258, -5873, 6684], [18181, -14719, 6396], [13775, -3160, 12790], [-25956, -20624, 10960], [12699, 17214, -13241], [29735, 2620, -2061], [-8583, 4558, 2569], [13452, 15445, -11294], [14809, -3321, 2], [17512, -7911, 18217], [-36464, 2550, 5773], [30339, 1853, -4991], [10421, 13375, 928], [-2034, -17088, 2590], [33628, 4448, 537], [24927, -4927, 5071], [-1061, 398, -13543], [-9238, -5637, -14518], [27648, 4097, 8544], [-17053, 151, -11020], [-16844, 13840, 741], [29082, -12786, -7028], [9434, -20397, 2133], [24809, -8229, -5659], [15894, 189, -4311], [12868, 14240, 762], [-41811, 8353, 457], [9536, -33537, -1940], [16488, -13407, -10421], [76179, -446, 2908], [-337, 5304, -20326], [2104, 12986, 3573], [-8389, -3153, 15669], [4378, 5514, 959], [-339, -2454, -17923], [14525, -1037, -13937], [23114, -22494, 11304], [-28469, -5374, -21210], [-40244, 10634, 1976], [15611, 20278, -3262], [-5938, 8235, -13996], [19331, 10052, 198], [19393, 6203, 18284], [1081, -2686, -8394], [-13190, 16333, -3014], [-58047, 6133, -9590], [-43567, 25812, 11428], [-9433, -32898, -493], [41991, 10218, 5077], [-30783, -4791, -16199], [-70002, -16594, 3316], [15662, 12866, 457], [21996, -16692, 10359], [7046, -32857, -1967], [-16422, 4339, 659], [32047, 20886, 4928], [7254, 5153, -7026], [-38256, 20199, 14921], [40999, -18148, 4533], [-57239, -18090, 13234], [-21978, -11356, -3268], [63458, -5227, -5259], [32864, 9936, 12460], [-16365, 8647, -13044], [-19895, 23678, 13768], [14575, 16185, -19777], [46502, -18606, -9557], [-14826, -11607, -9814], [-24147, -4283, 15272], [-53526, 4191, 9805], [-20462, -16616, 4043], [-12685, 3976, -13457], [-32800, -12536, -11792], [-6518, -42879, -3383], [-20892, -12665, -5757], [24453, 45462, -19351], [16694, 6759, 27044], [2087, 4141, 6018], [4563, -36750, -10447], [-20912, 23112, -24213], [-90145, 11473, -845], [64280, -7067, 14385], [-22336, 22910, -18637], [-31244, 11347, -11073], [-55923, 18213, -18280], [-15759, 18871, 6443], [-65140, 12436, -1508], [140, 24070, 5021], [-300, -19967, -19597], [-31744, 24078, -8920], [13587, -14619, 31708], [-12827, 5988, -22598], [11855, -4336, -4622], [8608, 27560, 8436], [-49395, 1896, -7161], [-34198, 4370, 23900], [-3831, -36756, 6150], [-6574, -1096, 18572], [76977, -9581, 15571], [-373, 13313, 22055], [-11512, 1405, -5163], [36990, 51705, -9515], [-15229, -24204, -28452], [-6856, 44554, 14699], [-5001, 11336, 34176], [3754, -12647, 11125], [-90937, -4561, 5426], [-30716, 30650, -5565], [-10485, -24720, -71], [26937, -5624, 21349], [64616, -37913, -6419], [-2433, -283, 13984], [-88309, 20579, 9666], [22007, 31558, -13801], [-19816, 37476, -162], [100178, 31983, -18931], [-881, 18570, -7756], [-12773, 11936, 22738], [13962, 30129, -37053], [70778, 2906, -5474], [-74340, -308, 13425], [5296, -9121, 11210], [-39718, -18327, -26089], [-115262, 2326, 1816], [46262, -32299, -3265], [71326, 6153, 7356], [45163, -22509, 13384], [-50941, 32508, -19283], [-12434, -7700, 15038], [-40574, -11069, 30683], [-23339, -31611, 23828], [-49144, 2893, 8989], [42244, -8518, -3200], [-63408, -2254, -13000], [-41299, 1712, -39829], [65363, 2312, 24007], [-9894, 12332, 40144], [-25552, 42541, -29779], [-65379, -31493, -3190], [97128, -3072, -2617], [96340, -18293, 25659], [-10422, -25602, 6280], [-82735, -30956, 25078], [-63387, 20911, -11738], [-46362, -16100, -38107], [26042, 19281, -12413], [93853, -29622, -24176], [66255, 40585, -823], [30208, 71522, -25386], [-77163, -65230, 26667], [-51612, -51662, 13600], [-19965, -18312, -33607], [-6680, 36468, -29094], [4155, 14085, 36988], [-77233, 13037, 25046], [-52671, -5143, 12281], [33599, -30046, -1987], [2236, 48642, 21547], [-81012, -4851, -20081], [1281, -19332, -10577], [-103888, 2085, -15849], [63958, -36115, 10996], [-71714, -58393, 31752], [-79492, -27913, 5981], [16311, 23028, -34383], [-10599, -24843, -11653], [17128, 58204, -2918], [88999, 19121, -19334], [23350, -4825, 48952], [-98838, 37938, -22589], [-65532, -15436, -13895], [-10772, 26556, 9528], [-46980, 9545, 5803], [14042, 9700, 38242], [-104515, 10799, -21580], [-7580, 23567, -16490], [-73197, -69756, 32720], [-50615, 2001, 38685], [40555, 52657, -26679], [-32453, -43133, -16202], [-106836, 39907, 25640], [34495, 11519, -10334], [-24901, -17286, 36349], [-7525, -9183, 10207], [96221, -41240, 861], [-68142, -62766, 39569], [-136111, -20684, -761], [-42239, -17485, 10806], [-14172, 60981, -27889], [-55707, 22095, -70734], [-5679, 48349, 28442], [63555, -72516, -18750], [126643, 51278, -17933], [-90704, 23551, -51647], [-17905, 38243, 16958], [104783, -39229, 12452], [-72128, 64718, 23672], [67248, -57637, -28403], [125441, 4033, 369], [75660, -65706, 5530], [-44224, 4785, -63089], [-13150, -52724, -22674], [151443, -5026, -2552], [-32284, -18241, -30910], [-148640, 47401, 6289], [-38177, 51629, 41595], [1260, -58686, 28006], [21697, 36910, 15564], [9158, 66071, -17803], [-134549, 43570, -17005], [56806, -16321, 24094], [-25022, 86522, 3258], [47138, -32423, -3598], [94363, -26596, -34434], [-36141, -46704, 24075], [2640, 62903, 36373], [-49829, -20252, -33605], [251423, -7940, -4337], [-57779, 58216, 4576], [50338, 24321, 3671], [20346, -73473, 20911], [-38732, 65209, 23353], [16950, 80152, -36891], [64566, 27939, 49009], [209547, -12582, -10928], [27725, -47889, 36597], [60233, 6580, -26554], [-3721, 26536, 36671], [24823, -23439, -27218], [36731, -14510, 35745], [9744, -36595, 9837], [62243, -67112, -17176], [90801, 38655, 28380], [-28842, 106657, -12337], [-41955, -112283, 41039], [13208, -17402, -23984], [37765, -86261, -38510], [59370, -57795, -10645], [-3145, 109570, -15580], [46193, 14836, 18657], [94317, 423, 43645], [-8199, -58396, 8018], [-48189, 10935, -30424], [-87576, 44019, -53197], [-68090, 26353, -58439], [-15091, -3655, -25437], [72208, 11295, 22423], [149212, -50227, 14451], [-140478, 20187, -25776], [50438, -6143, -14069]]
